//VertexShader
#version 430 core
layout(location = 0) in vec3 aPos; // the position of the vertex
layout (location = 4) in vec3 aNormal; // the normal of the vertex
layout (location = 5) in vec3 aUvTexture; // the texture UV coordinate of the vertex
layout (location = 7) in vec3 aTangent; // the tangent variable to get the normals from normal mapping to tangent space

layout(location = 1) uniform mat4 model; // the model matrix
layout(location = 2) uniform mat4 view; // the view matrix
layout(location = 3) uniform mat4 projection; // the projection matrix
layout(location = 23) uniform vec3 cameraPos; // the camera position
layout(location = 24) uniform float wavelength;
layout(location = 25) uniform float peak;
layout(location = 26) uniform float time;
layout(location = 27) uniform int water;

const float pi = 3.14159265;

out vec3 FragPos;
out vec3 Normal;
out vec3 viewDir;
out vec2 TexCoords;
out vec2 tectureCoords;
out mat3 TBN;

void main()
{
    // transfer the position to world coordinate space
	vec4 position_world = model * vec4(aPos, 1.0);
	FragPos = position_world.xyz;

    // transfer the normal to world coordinate space
	Normal = mat3(transpose(inverse(model))) * aNormal;

    // calculate the viewing direction
	viewDir = normalize(cameraPos - FragPos);

    // get the texture coordinate from the UV coordinate
	TexCoords = abs(vec2(aUvTexture.x, aUvTexture.y) - vec2(1.0f, 1.0f));

	// calculate aBitangent from aNormal and aTangent
	vec3 aBitangent = cross(aNormal, aTangent);

	// bring tangent, bitangent and normal to world space coordinates
	vec3 T = normalize(vec3(model * vec4(aTangent,   0.0)));
	vec3 B = normalize(vec3(model * vec4(aBitangent, 0.0)));
	vec3 N = normalize(vec3(model * vec4(aNormal,    0.0)));
	TBN = mat3(T, B, N);

	if (water == 1) {
	    float a = aPos.x;
		float b = peak - 1.0f;
		float k = (2.0f * pi) / wavelength;
		float c = sqrt(9.81f/k);

		float wave_dx = (exp(k * b)/ k) * sin(k * (a + c * time));
		float wave_dy = (-exp(k * b)/ k) * cos(k * (a + c * time));    

		gl_Position = projection * view * model * vec4( aPos.x, aPos.y + wave_dy/4, aPos.z, 1.0f);
	} else {
		gl_Position = projection * view * position_world;
	}
}